/************************************************************************/
/*																		*/
/*	DPG1.cpp		--		Definition for DPG1 library 	    		*/
/*																		*/
/************************************************************************/
/*	Author:		Eric Marsh												*/
/*	Copyright 2016, Digilent Inc.										*/
/************************************************************************/
/*  File Description:													*/
/*		This file defines functions for the PmodDPG1					*/
/*																		*/
/************************************************************************/
/*  Revision History:													*/
/*																		*/
/*	06/23/2016(EricM): created											*/
/*																		*/
/************************************************************************/


/* ------------------------------------------------------------ */
/*				Include File Definitions						*/
/* ------------------------------------------------------------ */
#include "DPG1.h"
#include <DSPI.h>

/* ------------------------------------------------------------ */
/*        DPG1::DPG1
**
**        Synopsis:
**				
**        Parameters:
**
**
**
**        Return Values:
**                void 
**
**        Errors:
**
**
**        Description:
**			Class constructor. Performs variables initialization tasks
**
**
*/
DPG1::DPG1()
{
	pdspi = NULL;
	
}


/* ------------------------------------------------------------ */
/*        DPG1::begin
**
**        Synopsis:
**				myDPG1.begin(PAR_ACCESS_SPI0);
**        Parameters:
**				uint8_t bAccessType	- the SPI interface where the Pmod is connected. It can be one of:
**					0	PAR_ACCESS_SPI0	- indicates SPI port 2, connetor JB
**					1	PAR_ACCESS_SPI1	- indicates SPI port 1, connector J1
**
**				uint32_t spd - The frequency at which the SPI clock should be set to. 1 Mhz by default
**   			uint8_t pType - The unit type the pressure level should be returned in. kPa by default.
**
**        Return Values:
**                void 
**
**        Errors:
**
**
**        Description:
**				This function initializes the specific SPI interface used, setting the SPI frequency to a default value of 1 MHz.
**
**
*/
void DPG1::begin(uint8_t bAccessType, uint32_t spd, uint8_t pType)
{
	if(bAccessType == PAR_ACCESS_SPI0)
	{
		pdspi = new DSPI0();
		m_SSPin = PIN_DSPI0_SS;	// default SS pin for SPI0
	}
	if(bAccessType == PAR_ACCESS_SPI1)
	{
		pdspi = new DSPI1();
		m_SSPin = PIN_DSPI1_SS;	// default SS pin for SPI1
	}
	if(pdspi != NULL)
	{
		pdspi->begin(m_SSPin);
		pdspi->setMode(DSPI_MODE0);
		pdspi->setSpeed(spd);
		default_type = pType;
	}
}

/* ------------------------------------------------------------ */
/*        DPG1::ReadNozzle
**
**        Synopsis:
**				myDPG1.ReadNozzle();
**        Parameters:
**				None
**        Return Values:
**                int - 12 bit value read from PmodDPG1 
**
**        Errors:
**
**
**        Description:
**				This function pulls pressure level data from PmodDPG1, 
**				configures them, and returns a 12 bit integer indicating the pressure.
**				Even without applying any differential pressure, the sensor will still
**				return a small decimal number close to zero.
**
*/
int DPG1::ReadNozzle(){

	int upper_byte = 0;
	int lower_byte = 0;
	int combination = 0;
	  
	//collect the 12 bits of data in two bytes
	digitalWrite(m_SSPin, LOW);
	upper_byte = pdspi->transfer(0);
	lower_byte = pdspi->transfer(0);
	digitalWrite(m_SSPin, HIGH);
	  
	//combine them into a single variable
	combination = (upper_byte<<8 | lower_byte);
	 
	//remove the upper 4 bits since we are receiving a 12 bit value
	combination = (combination & 0b0000111111111111);
	
	// return the result
	return combination;
}

/* ------------------------------------------------------------ */
/*        DPG1::GetPressure
**
**        Synopsis:
**				myDPG1.GetPressure();
**        Parameters:
**				int pType (optional, 0 by default, meaning kPa) - the pressure unit the function should return
**        Return Values:
**                double - the pressure data after it's been calculated
**
**        Errors:
**
**
**        Description:
**				This function takes the data from the Pmod, and calculates its pressure in a variety
**				of pressure units. By default, it will return pressure in units of kPa. Even without 
**				applying any differential pressure, the sensor will still return a small decimal
**				number close to zero.
**
**
*/
double DPG1::GetPressure(int pType){

	int type;
	double kPa;
	
	// checks for optional parameter
	if(pType != -1)
	{
		int type = pType;
	}
	else
	{
		int type = default_type;
	}
	
	// calculate kPa from raw data
	kPa = ((double)ReadNozzle()/4096.0 - 0.08)/0.09;
	
	switch(type)
	{
		// Kilopascal
		case 0: 
			return kPa;
		// Atm
		case 1: 
			return kPa/101.325;
		// PSI
		case 2: 
			return kPa*0.145038;
		// InH20
		case 3: 
			return kPa*4.01865;
		// CmH20	
		case 4: 
			return kPa*10.1972;
		// MmHg
		case 5: 
			return kPa*760.0/101.325;
		default:
			return kPa;
	}
}
